'''**************************** Module Header ******************************\
'''* Module Name:    WorkerRole.vb
'''* Project:        MSDNArduinoListener
'''* 
'''* This source is subject to the Microsoft Public License.
'''* See http://www.microsoft.com/en-us/openness/licenses.aspx#MPL.
'''* All other rights reserved.
'''* 
'''* THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND, 
'''* EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED 
'''* WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR PURPOSE.
'''\****************************************************************************

Imports System.Net
Imports System.Threading
Imports Microsoft.WindowsAzure
Imports Microsoft.WindowsAzure.Diagnostics
Imports Microsoft.WindowsAzure.ServiceRuntime
Imports Microsoft.WindowsAzure.Storage
Imports Microsoft.ServiceBus.Messaging
Imports System.Threading.Tasks
Imports System.Net.Sockets
Imports System.IO
Imports Microsoft.ServiceBus

Namespace ArduinoListener
	Public Class WorkerRole
		Inherits RoleEntryPoint
		Public Overrides Sub Run()
			Trace.WriteLine("Starting Application", "Information")

			Dim deviceEp = RoleEnvironment.CurrentRoleInstance.InstanceEndpoints("arduinoEP")
            Dim messagingFactoryVar = MessagingFactory.CreateFromConnectionString(RoleEnvironment.GetConfigurationSettingValue("serviceBusConnectionString"))

            SwitchServer.Run(deviceEp.IPEndpoint, messagingFactoryVar)
		End Sub

		Public NotInheritable Class SwitchServer
			Private Shared timeout As Integer = 10000
			Private Shared ReadOnly OnFrame() As Byte = { &H1 }
			Private Shared ReadOnly OffFrame() As Byte = { &H2 }

			Public Shared Sub Run(ByVal deviceEndPoint As IPEndPoint, ByVal messagingFactory As MessagingFactory)
				Trace.WriteLine("Starting Server", "Information")

				Task.WaitAll( { RunDeviceEndpoint(deviceEndPoint, messagingFactory) })
			End Sub

			Private Shared async Function RunDeviceEndpoint(ByVal deviceEP As IPEndPoint, ByVal messagingFactory As MessagingFactory) As Task
				Dim deviceServer = New TcpListener(deviceEP)
				deviceServer.Start(10)

				Trace.WriteLine("Listening for connections", "Information")

				Try
					Do
                        Dim connection As TcpClient = Await deviceServer.AcceptTcpClientAsync
						If connection IsNot Nothing Then
							Try
								connection.NoDelay = True
								connection.ReceiveTimeout = timeout
                                Dim deviceConnectionStream As NetworkStream = connection.GetStream
								Dim readBuffer = New Byte(63){}

								If await deviceConnectionStream.ReadAsync(readBuffer, 0, 4) = 4 Then
									Dim deviceId As Integer = IPAddress.NetworkToHostOrder(BitConverter.ToInt32(readBuffer, 0))
									Trace.WriteLine("Received signal for device:" & deviceId, "Information")

									Try
										'Create device queue if it doesn't exist
                                        Dim namespaceManagerVar = NamespaceManager.CreateFromConnectionString(RoleEnvironment.GetConfigurationSettingValue("serviceBusConnectionString"))
                                        If Not namespaceManagerVar.QueueExists(String.Format("dev{0:X8}", deviceId)) Then
                                            namespaceManagerVar.CreateQueue(String.Format("dev{0:X8}", deviceId))
                                        End If

										Console.WriteLine(String.Format("dev{0:X8}", deviceId), "Information")
										' set up the receiver for the per-device queue and sender for the response queue
										Dim deviceQueueReceiver = messagingFactory.CreateMessageReceiver(String.Format("dev{0:X8}", deviceId), ReceiveMode.PeekLock)

										Trace.WriteLine("Connected to Queue")

										Do
											' receive from the queue
											Trace.WriteLine("Reading the Queue", "Information")
											Dim message As BrokeredMessage = Nothing
                                            message = Await deviceQueueReceiver.ReceiveAsync

											Try
												If message IsNot Nothing Then
													Dim stream As Stream = message.GetBody(Of Stream)()
													Dim reader As New StreamReader(stream)
                                                    Dim command As String = reader.ReadToEnd

													If command IsNot Nothing Then
                                                        Select Case command.ToUpperInvariant
                                                            Case "ON"
                                                                Await deviceConnectionStream.WriteAsync(OnFrame, 0, OnFrame.Length)
                                                                Await message.CompleteAsync
                                                            Case "OFF"
                                                                Await deviceConnectionStream.WriteAsync(OffFrame, 0, OffFrame.Length)
                                                                Await message.CompleteAsync
                                                        End Select
													End If
												End If
											Catch e As Exception
												Trace.TraceError(e.Message)
												deviceQueueReceiver.Close()
											End Try
										Loop While True
									Catch e As Exception
										Trace.TraceError(e.Message)
									Finally
										connection.Close()
									End Try
								Else
									connection.Close()
								End If
							Catch se As SocketException
								' log
								Trace.TraceError(se.Message)
								connection.Close()
							Catch ioe As IOException
								Trace.TraceError(ioe.Message)
								connection.Close()
							Catch e As Exception
								Trace.TraceError(e.Message)
							End Try
						End If
					Loop While True
				Catch e As Exception
					Trace.TraceError(e.Message)
				Finally
					deviceServer.Stop()
				End Try
			End Function
		End Class
	End Class
End Namespace
